<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:l="http://www.landxml.org/schema/LandXML-1.2"
                xmlns:h="http://xml.hexagon.com/schema/HeXML-1.6"
                xmlns:r="urn:swdc:resources"
                xmlns="http://www.landxml.org/schema/LandXML-1.2"
                xmlns:mat="urn:mathematics"
                xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                exclude-result-prefixes="h  r mat">

  <!-- **************************************************************************************** -->
  <!-- Source template ID: {} -->
  <!-- Source version    : 1.0 -->
  <!-- Description       : APA Format -->
  <!-- **************************************************************************************** -->
  
  <xsl:output method="text" indent="no" encoding="utf-8"/> 
  <xsl:variable name="fileExt" select="'APA'"/>
  <xsl:variable name="fileDesc" select="'APA Format'"/>

  <!-- Colums separators for the ASCII file -->
  <xsl:variable name="ColumnsSep" select="','"/>
  
  <!-- Set the numeric display details i.e. decimal point, thousand separator etc -->
  <xsl:variable name="DecimalSep" select="'.'"/>

  <!-- Configure the default values for distances and angles values -->
  <xsl:variable name="FormatDistance" select ="$Decimals3"/>
  <xsl:variable name="FormatAngle" select ="$Decimals4"/>

  <!-- Default value when number does not exist -->
  <xsl:decimal-format NaN=""/>

  <!-- Predefined format values -->
  <xsl:variable name="Decimals0" select="string('#0')"/>
  <xsl:variable name="Decimals1" select="string(concat('#0', $DecimalSep, '0'))"/>
  <xsl:variable name="Decimals2" select="string(concat('#0', $DecimalSep, '00'))"/>
  <xsl:variable name="Decimals3" select="string(concat('#0', $DecimalSep, '000'))"/>
  <xsl:variable name="Decimals4" select="string(concat('#0', $DecimalSep, '0000'))"/>
  <xsl:variable name="Decimals5" select="string(concat('#0', $DecimalSep, '00000'))"/>
  
  <!-- Global variables -->
  <xsl:variable name="APAversion" select ="02"/>

                            
  <!-- XML processing -->
  <xsl:template match="/">
  
    <!--set Units-->
    <!--imperial-->
    <xsl:variable name="Imperial" select="// l:Imperial"/>
    <xsl:choose>
	  <xsl:when test="$Imperial">
	    <xsl:variable name ="Unit" select ="mat:setXmlUnits(string($Imperial/@linearUnit), string($Imperial/@angularUnit), string($Imperial/@temperatureUnit),string( $Imperial/@pressureUnit))"/>
	  </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="Metric" select="// l:Metric"/>
        <xsl:variable name ="Unit" select ="mat:setXmlUnits(string($Metric/@linearUnit), string($Metric/@angularUnit), string($Metric/@temperatureUnit),string($Metric/@pressureUnit))"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <!-- A Block -->
    <xsl:call-template name="APA_A"/>
    <xsl:call-template name="NewLine"/>
    
    <!-- Go through all surveys if different totalstations had been used-->
    <xsl:apply-templates select="//l:Survey"/>
    
    <!-- Z Block -->
    <xsl:call-template name="APA_Z"/>
    
  </xsl:template>

  <!-- Key definitions-->
  <!--<xsl:key name="KeyInstrumentSetup" match="l:InstrumentSetup" use="@id"/>
  <xsl:key name="KeyCgPoint" match="l:CgPoint" use="@name"/>-->
  <xsl:key name="Point" match="h:Point" use="@uniqueID"/>
  <xsl:key name="Survey" match="h:Survey" use="@name"/>
  <xsl:key name="TPSCorrections" match="h:TPSCorrection" use="@uniqueID"/>
  <xsl:key name="RawObservation" match="h:RawObservation" use="@targetPntRef"/>
  <xsl:key name="LRawObservation" match="l:RawObservation" use="l:TargetPoint/@pntRef"/>
  <xsl:key name="InstrumentSetup" match="l:InstrumentSetup" use="@id"/>
  <xsl:key name="ObservationGroup" match="l:ObservationGroup" use="@id"/>
  <xsl:key name="ApplicationSetup" match="h:ApplicationTPSSetupResults" use="@TPSSetupUniqueID"/>
  
  <xsl:template match="l:Survey">
    
    <xsl:variable name="Job_Name">
	  <xsl:call-template name="Job_Name">
		<xsl:with-param name="Name" select="l:SurveyHeader/@name"/>
		</xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="Instrument_Name" select="l:Equipment/l:InstrumentDetails /@model"/>
    <xsl:variable name="Instrument_SrNr" select="l:Equipment/l:InstrumentDetails /@serialNumber"/>
    
    <!--hexml survey element operator -->
    <xsl:variable name="Survey" select="key('Survey', l:SurveyHeader/@name)"/> 

    <xsl:if test="$Instrument_Name">
    
      <!--output line B-->
      <xsl:call-template name="APA_B">
        <xsl:with-param name="Job_Name" select="$Job_Name"/>
        <xsl:with-param name="Instrument_Name" select="$Instrument_Name"/>
        <xsl:with-param name="Instrument_SrNr" select="$Instrument_SrNr"/>
      </xsl:call-template>
      <xsl:call-template name="NewLine"/>

      <xsl:apply-templates select="key('Survey', l:SurveyHeader/@name)"/> 
    </xsl:if>       
  
  </xsl:template>
  
  <!-- RawObservations nodes processing-->
  <xsl:template match="l:InstrumentSetup">

    <xsl:param name="TpsCorrectionRef"/>
    <xsl:variable name="Duration" select="substring(l:InstrumentPoint/@timeStamp, 12, 5)"/> <!--todo Starttime - end time-->
    <xsl:variable name="ApplicationSetup" select="key('ApplicationSetup', @id)"/>
    <!--<xsl:variable name="StartTime" select="substring($ApplicationSetup/@ApplicationStartDateTime, 12, 5)"/>
    <xsl:variable name="EndTime" select="substring($ApplicationSetup/@ApplicationEndDateTime, 12, 5)"/>-->
    <xsl:variable name="ObservationGroup" select="key('ObservationGroup', concat('o', @id))"/>
    <!--<xsl:variable name="NumPoints" select="count($ObservationGroup/l:Backsight)"/>-->
    <xsl:variable name="ID" select="@id"/>
    <xsl:variable name="RawObservationArray" select="//h:InstrumentSetup[@uniqueID = $ID]/h:RawObservation[@tpsCorrectionRef = $TpsCorrectionRef]"/>
    <xsl:variable name="NumPoints" select="count($RawObservationArray)"/>
    <!--get the fist observation-->
    <xsl:variable name="LStartObservation" select="key('LRawObservation', $RawObservationArray[1]/@targetPntRef )"/>
    <xsl:variable name="StartTime" select="substring($LStartObservation/@timeStamp, 12, 5)"/>
    <xsl:variable name="LEndObservation" select="key('LRawObservation', $RawObservationArray[last()]/@targetPntRef )"/>
    <xsl:variable name="EndTime" select="substring($LEndObservation/@timeStamp, 12, 5)"/>
  
     <!--output line D-->
    <xsl:call-template name="APA_D">
      <xsl:with-param name="StationName" select="@stationName"/>
      <xsl:with-param name="Comment" select="''"/>
      <xsl:with-param name="InstrumentHeight" select="mat:toMeters(number(@instrumentHeight))"/>
      <xsl:with-param name="StationShift" select="'0'"/>
      <xsl:with-param name="NumHzSet" select="'1'"/>
      <xsl:with-param name="NumVSet" select="'1'"/>
      <xsl:with-param name="NumSdSet" select="'1'"/>
      <xsl:with-param name="Duration" select="$Duration"/>
      <xsl:with-param name="NumPoints" select="$NumPoints"/>
      <xsl:with-param name="StartTime" select="$StartTime"/>
      <xsl:with-param name="EndTime" select="$EndTime"/>
        
    </xsl:call-template>
    <xsl:call-template name="NewLine"/>
     
     <!--output line E--> 
      <xsl:variable name="StationName" select="@stationName"/>    
      <xsl:variable name="RedoingSetup" select="count(//l:InstrumentSetup[@stationName = $StationName])"/>      
      <!--<xsl:choose>
        <xsl:when test="$RedoingSetup = '1'">-->
          <!--always one setup ( current node ) has been there-->
           <xsl:call-template name="APA_E">
            <xsl:with-param name="Redoing" select="'0'"/>
          </xsl:call-template>        
        <!--</xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="APA_E">
            <xsl:with-param name="Redoing" select="'1'"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>-->
      <xsl:call-template name="NewLine"/>

    <xsl:apply-templates select="$ObservationGroup/l:RawObservation">
      <xsl:sort select="@timeStamp" order="ascending"/>
      <xsl:with-param name="TpsCorrectionRef" select="$TpsCorrectionRef"/>
    </xsl:apply-templates>
  </xsl:template>

  <!-- RawObservations nodes processing-->
  <xsl:template name="RawObservation" match="l:RawObservation">
  
  <xsl:param name="TpsCorrectionRef"/>
    
    <!-- Reflector const for current RawObservation -->
    <xsl:variable name="RawObservation" select="key('RawObservation', ./l:TargetPoint/@pntRef)"/>
    <xsl:variable name="Increment" select="9000 + position()"/>    
    
    <xsl:if test="$TpsCorrectionRef = $RawObservation/@tpsCorrectionRef"   >
    
      <!--output line F-->
      <xsl:choose>
		  <xsl:when test="@directFace = 'true' ">
		    <xsl:call-template name="APA_F">
              <xsl:with-param name="BacksightName" select="./l:TargetPoint/@name"/> <!--todo A1 is missing-->
              <xsl:with-param name="TargetHgt" select="mat:toMeters(number(@targetHeight))"/>
              <xsl:with-param name="Increment" select="concat('r', $Increment)"/>
              <xsl:with-param name="Hz" select="mat:toDMS(number(@horizAngle),4)"/>
              <xsl:with-param name="V" select="mat:toDMS(number(@zenithAngle),4)"/>
              <xsl:with-param name="Sd" select="mat:toMeters(number(@slopeDistance))"/>
              <xsl:with-param name="PrismConst" select="mat:toMillimeters(number($RawObservation/@reflectorConstant))"/>        
            </xsl:call-template>
		  </xsl:when>
		  <xsl:otherwise>
		    <xsl:call-template name="APA_F">
              <xsl:with-param name="BacksightName" select="./l:TargetPoint/@name"/> <!--todo A1 is missing-->
              <xsl:with-param name="TargetHgt" select="mat:toMeters(number(@targetHeight))"/>
              <xsl:with-param name="Increment" select="concat('l', $Increment)"/>
              <xsl:with-param name="Hz" select="mat:toDMS(number(@horizAngle),4)"/>
              <xsl:with-param name="V" select="mat:toDMS(number(@zenithAngle),4)"/>
              <xsl:with-param name="Sd" select="mat:toMeters(number(@slopeDistance))"/>
              <xsl:with-param name="PrismConst" select="mat:toMillimeters(number($RawObservation/@reflectorConstant))"/>
            </xsl:call-template>
		  </xsl:otherwise>
	  </xsl:choose>
      <xsl:call-template name="NewLine"/>
    </xsl:if>
    
  </xsl:template>
  
  <xsl:template match="h:Survey">
    <xsl:apply-templates select="h:TPSCorrection">
      <xsl:sort select="@uniqueID" order="ascending"/>
    </xsl:apply-templates>  
  </xsl:template>
  
  <xsl:template match="h:TPSCorrection">  
    <!--array with TPS Raw observation that are using this corrections-->
    <xsl:variable name="UniqueID" select="@uniqueID"/>
    <!--<xsl:variable name="SetupArray" select="//h:InstrumentSetup[not(/h:RawObservation[@tpsCorrectionRef = $UniqueID].count()=0)]"/>-->
    <xsl:variable name="RawObservationArray" select="//h:RawObservation[@tpsCorrectionRef = $UniqueID]"/>
    <!--check if there are more than on point in the array-->
     <xsl:if test="$RawObservationArray">
  
      <!-- variables for the C Record -->
      <!--<xsl:variable name="Weather" select="$Point/h:PointCode[1]/h:Attribute[1]/@value"/>
      <xsl:variable name="Wind" select="$Point//h:PointCode//h:Attribute[2]/@value"/>-->
      <xsl:variable name="Temp" select="mat:toCelsius(number(@dryTemperature))"/>
      <xsl:variable name="Pressure" select="mat:toHPascal(number(@pressure))"/>
      <xsl:variable name="PPM" select="@atmosphericPPM"/> 
	
      <!--output line C-->
      <xsl:call-template name="APA_C">
	    <!--<xsl:with-param name="Weather" select="$Weather"/>
	    <xsl:with-param name="Wind" select="$Wind"/>-->
	    <xsl:with-param name="Weather" select="1"/>
	    <xsl:with-param name="Wind" select="1"/>
	    <xsl:with-param name="Temp" select="$Temp"/>
	    <xsl:with-param name="Pressure" select="$Pressure"/>
	    <xsl:with-param name="PPM" select="$PPM"/>
      </xsl:call-template>
      <xsl:call-template name="NewLine"/>
      
      <!--go through all setups-->
      <xsl:apply-templates select="../h:InstrumentSetup">
        <xsl:with-param name="TpsCorrectionRef" select="@uniqueID"/>
      </xsl:apply-templates>
      
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="h:InstrumentSetup">
    <xsl:param name="TpsCorrectionRef"/>
    <xsl:variable name="RawObservationArray" select="h:RawObservation[@tpsCorrectionRef = $TpsCorrectionRef]"/>
    <xsl:if test="$RawObservationArray">
      <xsl:apply-templates select="key('InstrumentSetup', @uniqueID)">
        <xsl:with-param name="TpsCorrectionRef" select="$TpsCorrectionRef"/>
      </xsl:apply-templates>
    </xsl:if>
    
  </xsl:template>
  
  <!-- APA A line -->
  <!-- A, APA Version, LeicaTPS Fixed, Firmware Version , Checksum -->
  <xsl:template name="APA_A">
    <xsl:variable name="string" select="concat('A', $ColumnsSep, '02', $ColumnsSep,'LeicaTPS', $ColumnsSep, // l:Application/@version, $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
     <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- APA B line -->
  <!-- B, Jobname, Instrument Name, Serial number, Date of output , Checksum -->
  <xsl:template name="APA_B">
    <xsl:param name="Job_Name"/>
    <xsl:param name="Instrument_Name"/>
    <xsl:param name="Instrument_SrNr"/>
    
    <xsl:variable name="OldestCgPointTime">
      <xsl:for-each select="//l:CgPoint">
        <xsl:sort select="@timeStamp" order="ascending"/>
        <xsl:if test="position()=1">
          <xsl:value-of select="@timeStamp"/>
        </xsl:if>   
      </xsl:for-each>
    </xsl:variable>
      
    <xsl:variable name="Date" select="concat(substring($OldestCgPointTime, 3, 2), '/', substring($OldestCgPointTime, 6, 2), '/', substring($OldestCgPointTime, 9, 2))"/>
	
    <xsl:variable name="string" select="concat('B', $ColumnsSep, $Job_Name, $ColumnsSep, $Instrument_Name, $ColumnsSep, $Instrument_SrNr, $ColumnsSep, $ColumnsSep, $Date, $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
      <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>    
    
  <!-- APA C line -->
  <!-- C, weather, wind, temperature , preassure, ppm, 1, Checksum -->
  <xsl:template name="APA_C">

    <xsl:param name="Weather"/>    
    <xsl:param name="Wind"/>
    <xsl:param name="Temp"/>
    <xsl:param name="Pressure"/>
    <xsl:param name="PPM"/>

    <xsl:variable name="string" select="concat('C', $ColumnsSep, $Weather, $ColumnsSep, $Wind, $ColumnsSep, format-number($Temp, $Decimals1) , $ColumnsSep, format-number($Pressure, $Decimals1), $ColumnsSep, format-number($PPM, $Decimals1), $ColumnsSep, '1', $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
     <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>    
  
  <!-- APA D line -->
  <!-- D, Station Name, comment, Instrument height , shift station, number HZ set, number V set, number dist set, measure time, number of ponts, start time, end time, Checksum -->
  <xsl:template name="APA_D">

    <xsl:param name="StationName"/>    
    <xsl:param name="Comment"/>
    <xsl:param name="InstrumentHeight"/>
    <xsl:param name="StationShift"/>
    <xsl:param name="NumHzSet"/>
    <xsl:param name="NumVSet"/>
    <xsl:param name="NumSdSet"/>
    <xsl:param name="Duration"/>
    <xsl:param name="NumPoints"/>
    <xsl:param name="StartTime"/>
    <xsl:param name="EndTime"/>

    <!--<xsl:variable name="string" select="concat('D', $ColumnsSep, $StationName, $ColumnsSep, $Comment, $ColumnsSep, format-number($InstrumentHeight, $Decimals3) , $ColumnsSep, $StationShift, $ColumnsSep, $NumHzSet, $ColumnsSep, $NumVSet, $ColumnsSep, $NumSdSet, $ColumnsSep, $Duration, $ColumnsSep, $NumPoints, $ColumnsSep, $StartTime, $ColumnsSep, $EndTime, $ColumnsSep)"/>-->
    <xsl:variable name="string" select="concat('D', $ColumnsSep, $StationName, $ColumnsSep, $Comment, $ColumnsSep, format-number($InstrumentHeight, $Decimals3) , $ColumnsSep, '0,9911', format-number($NumPoints,'000'), $ColumnsSep, $StartTime, $ColumnsSep, $EndTime, $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
     <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>    
  
  <!-- APA E line -->
  <!-- E, Redoing, Checksum -->
  <xsl:template name="APA_E">
    <xsl:param name="Redoing"/>
	
    <xsl:variable name="string" select="concat('E', $ColumnsSep, $Redoing, $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
      <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>    
  
  <!-- APA F line -->
  <!-- F, Backsight Name, A1, Target height , incrementation, HZ, V, SD, - , -, -, 000, PrismCons, Checksum -->
  <xsl:template name="APA_F">

    <xsl:param name="BacksightName"/>    
    <xsl:param name="TargetHgt"/>
    <xsl:param name="Increment"/>    
    <xsl:param name="Hz"/>
    <xsl:param name="V"/>
    <xsl:param name="Sd"/>
    <xsl:param name="PrismConst"/>

	<xsl:if test="$Sd">	  
	  <xsl:variable name="string" select="concat('F', $ColumnsSep, $BacksightName, $ColumnsSep, $ColumnsSep, format-number($TargetHgt, $Decimals3) , $ColumnsSep, $Increment, $ColumnsSep, format-number($Hz, $FormatAngle), $ColumnsSep, format-number($V, $FormatAngle), $ColumnsSep, format-number($Sd, $FormatDistance), $ColumnsSep, $ColumnsSep, $ColumnsSep, $ColumnsSep, '000', $ColumnsSep, format-number($PrismConst, $Decimals1), $ColumnsSep)"/>
	  <xsl:value-of select="$string"/>
	   <!-- checksum -->
	   <xsl:call-template name="CheckSum">
	   <xsl:with-param name="stringValue" select="$string"/>
	   </xsl:call-template>
	 </xsl:if>
	 <xsl:if test="not($Sd)">	  
	  <xsl:variable name="string" select="concat('F', $ColumnsSep, $BacksightName, $ColumnsSep, $ColumnsSep, format-number($TargetHgt, $Decimals3) , $ColumnsSep, $Increment, $ColumnsSep, format-number($Hz, $FormatAngle), $ColumnsSep, format-number($V, $FormatAngle), $ColumnsSep, format-number($Sd, $FormatDistance), $ColumnsSep, $ColumnsSep, $ColumnsSep, $ColumnsSep, '000', $ColumnsSep, $ColumnsSep)"/>
	  <xsl:value-of select="$string"/>
	   <!-- checksum -->
	   <xsl:call-template name="CheckSum">
	   <xsl:with-param name="stringValue" select="$string"/>
	   </xsl:call-template>
	 </xsl:if>
    
  </xsl:template>    
  
  <!-- APA Z line -->
  <!-- Z, Checksum -->
  <xsl:template name="APA_Z">
	
    <xsl:variable name="string" select="concat('Z', $ColumnsSep)"/>
    <xsl:value-of select="$string"/>
    
      <!-- checksum -->
    <xsl:call-template name="CheckSum">
      <xsl:with-param name="stringValue" select="$string"/>
    </xsl:call-template>
    
  </xsl:template>    
  
  <!-- Template for Job Namen without _xxx -->
<xsl:template name="Job_Name">
	<xsl:param name="Name"/>
	<xsl:param name="Charcter" select="'_'"/>
	<xsl:if test="contains($Name, $Charcter)">
		<xsl:value-of select="substring-before($Name, $Charcter)"/>
		<xsl:if test="contains(substring-after($Name, $Charcter), $Charcter)">
			<xsl:value-of select="$Charcter"/>
		</xsl:if>
		<xsl:call-template name="Job_Name">
			<xsl:with-param name="Name" select="substring-after($Name, $Charcter)"/>
			<xsl:with-param name="Charcter" select="$Charcter"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>
  
 <!--Calculate the schsum of an string-->
  <xsl:template name="CheckSum">
    <xsl:param name="stringValue"/>
    
    <xsl:variable name="CheckSum" select="mat:checkSum($stringValue)"/>
    <xsl:value-of select="concat($CheckSum, $ColumnsSep)"/>
    
  </xsl:template>
  
  <!--Line terminator -->
  <xsl:template name="NewLine">
    <xsl:text>&#13;&#10;</xsl:text>
  </xsl:template>
  
<!--
<xsl:template match="l:LandXML">
abc CRC:<xsl:value-of select="math:checkSum('abc')"/>
</xsl:template>
-->
<msxsl:script language="JavaScript" implements-prefix="mat">
<![CDATA[

// Checksum calculator
//
function checkSum(str) {
	var crc = 0;
	
	for( var i = 0, iTop = str.length; i < iTop; i++ ) {
	crc += str.charCodeAt(i);
	}
	crc = crc % 0x40+0x20;
	
	return String.fromCharCode(crc);
}


function round(value, exp) {
  if (typeof exp === 'undefined' || +exp === 0)
    return Math.round(value);

  value = +value;
  exp  = +exp;

  if (isNaN(value) || !(typeof exp === 'number' && exp % 1 === 0))
    return NaN;

  // Shift
  value = value.toString().split('e');
  value = Math.round(+(value[0] + 'e' + (value[1] ? (+value[1] + exp) : exp)));

  // Shift back
  value = value.toString().split('e');
  return +(value[0] + 'e' + (value[1] ? (+value[1] - exp) : -exp));
}

// this function only works with the MSXML....Looks like that Altova debugger and translator does not like JS Objects

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Units
//
var LinearUNITS = {
MILLIMETER: { XML_NAME: "millimeter", ABBR: "mm", FACTOR_INT: 1 / 1000.0 },
CENTIMETER: { XML_NAME: "centimeter", ABBR: "cm", FACTOR_INT: 1 / 100.0 },
METER: { XML_NAME: "meter", ABBR: "m", FACTOR_INT: 1.0 },
KILOMETER: { XML_NAME: "kilometer", ABBR: "km", FACTOR_INT: 1000.0 },
INCH: { XML_NAME: "inch", ABBR: "in", FACTOR_INT: 0.0254 },
FOOT: { XML_NAME: "foot", ABBR: "fti", FACTOR_INT: 0.3048 },
US_SURVEY_FOOT: { XML_NAME: "USSurveyFoot", ABBR: "fts", FACTOR_INT: 1200.0 / 3937.0 },
MILE: { XML_NAME: "mile", ABBR: "mile", FACTOR_INT: 1609.344 } 
}

var AngularUNITS = {
RADIANS: { XML_NAME: "radians", ABBR: "rad", FACTOR_INT: 1.0 },
GRADS: { XML_NAME: "grads", ABBR: "gon", FACTOR_INT: Math.PI / 200.0 },
DEG: { XML_NAME: "decimal degrees", ABBR: "deg", FACTOR_INT: Math.PI / 180.0 },
DMS: { XML_NAME: "decimal dd.mm.ss", ABBR: "dms", FACTOR_INT: 1.0 }  // Factor 1 used because it is converted in function to rad
}

var TemperatureUNITS = {
CELSIUS: { XML_NAME: "celsius", ABBR: "C", FACTOR_INT: 1.0 },
KELVIN: { XML_NAME: "kelvin", ABBR: "K", FACTOR_INT: 1.0 },
FAHRENHEIT: { XML_NAME: "fahrenheit", ABBR: "F", FACTOR_INT: 1.8 }
}

var PressureUNITS = {
HECTOPASCAL: { XML_NAME: "HPA", ABBR: "hPa", FACTOR_INT: 1.0 },
MILLIBAR: { XML_NAME: "milliBars", ABBR: "mbar", FACTOR_INT: 1.0 },
MILLIMERCURY: { XML_NAME: "mmHG", ABBR: "mmHG", FACTOR_INT: 1.0 / 0.75006150504 },
MILLIMETERMERCURY: { XML_NAME: "millimeterHG", ABBR: "mmHG", FACTOR_INT: 1.0 / 0.75006150504 },
INCHMERCURY: { XML_NAME: "inchHG", ABBR: "inHG", FACTOR_INT: 1.0 / 0.029529980514 },
INMERCURY: { XML_NAME: "inHG", ABBR: "inHG", FACTOR_INT: 1.0 / 0.029529980514 }
}

var gLinearUnits;
var gAngularUnits;
var gTemperatureUnits;
var gPressureUnits;

// set the units from the xml document
function setXmlUnits(linearUnit, angularUnit, temperatureUnit, pressureUnit) {
	for (var u in LinearUNITS) {
		var uMapped = LinearUNITS[u];
		if (uMapped.XML_NAME == linearUnit) {
			gLinearUnits = uMapped;
			break;
		}
	}
	for (var v in AngularUNITS) {
		var vMapped = AngularUNITS[v];
		if (vMapped.XML_NAME == angularUnit) {
			gAngularUnits = vMapped;
      break;
		}
	}
	for (var w in TemperatureUNITS) {
		var wMapped = TemperatureUNITS[w];
		if (wMapped.XML_NAME == temperatureUnit) {
			gTemperatureUnits = wMapped;
      break;
		}
	}
	for (var x in PressureUNITS) {
		var xMapped = PressureUNITS[x];
		if (xMapped.XML_NAME == pressureUnit) {
			gPressureUnits = xMapped;
      break;
		}
	}
	return '';
}

function getUnits() {
return gLinearUnits.XML_NAME;
}

function getUnitAbbr() {
	return gLinearUnits.ABBR;
}

// linear units conversions
function toMillimeters(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters * 1000;
}

function toCentimeters(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters * 100;
}

function toMeters(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters;
}

function toKilometers(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters / 1000;
}

function toInch(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters * 39.370;
}

function toUSFootInch(value) {
    var usFoot = toUSFoot(value);
    var feet = Math.floor(usFoot);
    var inches = Math.round((usFoot - feet) * 12);
    return feet + "'" + inches + '"';
}

function toUSFootInchEighthInch(value) {
    var usFoot = toUSFoot(value);
    var feet = Math.floor(usFoot);
    var dInches = (usFoot - feet) * 12;
    var inches = Math.floor(dInches);
    var dEighthInch = 32 * (dInches - inches);
    var eighthInch = Math.floor(dEighthInch);
    
    if ( dEighthInch - eighthInch >= 0.5 )
    {
        eighthInch++;
        if (eighthInch == 32)
        {
            eighthInch = 0;
            inches++;
            if (inches == 12)
            {
                inches = 0;
                feet++;
            }
        }
    }
    return feet + "'" + inches + " " + eighthInch + "/32";
}

function toIntFoot(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters * 3.280839895;
}

function toUSFoot(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters * (3937.0 / 1200.0);
}

function toUSMiles(value) {
	var meters = value * gLinearUnits.FACTOR_INT;
	return meters / 1609.344;
}

// angular units conversions
function toRadian(value) {
    if( gAngularUnits.ABBR == "dms"){
      // convert dms to deg
      value = value.toPrecision(15);
      var strDMS = value.toString();
      var i = strDMS.indexOf(".");
      var sec = strDMS.slice(i+3,i+5);      
      var deg = strDMS.slice(0,i)/1 + strDMS.slice(i+1,i+3)/60 + sec.concat(".",strDMS.slice(i+5))/3600;
      // set the value as radians
      value = deg * (Math.PI / 180.0);
    }
	var radians = value * gAngularUnits.FACTOR_INT;
	return radians;
}

function toGrad(value) {
    if( gAngularUnits.ABBR == "dms"){
      // convert dms to deg
      value = value.toPrecision(15);
      var strDMS = value.toString();
      var i = strDMS.indexOf(".");
      var sec = strDMS.slice(i+3,i+5);      
      var deg = strDMS.slice(0,i)/1 + strDMS.slice(i+1,i+3)/60 + sec.concat(".",strDMS.slice(i+5))/3600;
      // set the value as radians
      value = deg * (Math.PI / 180.0);
    }
	var radians = value * gAngularUnits.FACTOR_INT;
	if( radians == (2 * Math.PI) ){
	  radians = 0.0;
	 }
	 if( radians > (2 * Math.PI ) ){
	  radians -= 2 * Math.PI;
	 }
	  
	return radians * (200 / Math.PI);
}

function toDeg(value) {
    if( gAngularUnits.ABBR == "dms"){
      // convert dms to deg
      value = value.toPrecision(15);
      var strDMS = value.toString();
      var i = strDMS.indexOf(".");
      var sec = strDMS.slice(i+3,i+5);      
      var deg = strDMS.slice(0,i)/1 + strDMS.slice(i+1,i+3)/60 + sec.concat(".",strDMS.slice(i+5))/3600;
      // set the value as radians
      value = deg * (Math.PI / 180.0);
    }
	var radians = value * gAngularUnits.FACTOR_INT;
	if( radians == (2 * Math.PI) ){
	  radians = 0.0;
	 }
	 if( radians > (2 * Math.PI ) ){
	  radians -= 2 * Math.PI;
	 }
	return radians * (180 / Math.PI);
}

function toDMS(value, numDigits) {
  var deg;
  var min;
  var sec;
  if( numDigits < 4){
    numDigits = 4;
  }
  if( gAngularUnits.ABBR == "dms"){
    // convert dms to deg
    value = value.toPrecision(15);
    if( Math.abs(value) <= 1e-8 ){
      value = "0.00000000";
    }
    var strDMS = value.toString();
    var i = strDMS.indexOf(".");
    var sec = strDMS.slice(i+3,i+5);
    sec = sec.concat(".",strDMS.slice(i+5));
    min = strDMS.slice(i+1,i+3);
    deg = strDMS.slice(0,i);
  }
  else{
    // convert to deg
    var dd = toDeg(value);
    dd = dd.toPrecision(15);    
    deg = dd | 0; // truncate dd to get degrees
    var frac = Math.abs(dd - deg); // get fractional part
    min = (frac * 60) | 0; // multiply fraction by 60 and truncate
  
    sec = frac * 3600 - min * 60;
        
  }
  sec = round(sec, (numDigits-4) );
  min = round(min, 0 );
  deg = round(deg, 0 );
  
  if( sec >= 60 ){
    sec -= 60;
    min += 1;
  }
  if( min >= 60 ){
    min -= 60;
    deg += 1;
  }
  if( deg >= 360 ){
    deg -= 360;
  }
  
  if( min < 10 ){
      min = "0" + min }
  if( sec < 10 ){
      sec = "0" + sec;
  }
  var strSec = sec.toString();  
  sec = strSec.slice(0,2);
  if( numDigits > 4 ){  
    sec += strSec.slice(3, numDigits-2);
  }
      
	return deg +"." + min + sec;
}

// temperature units conversions
function toCelsius(value) {
  if( gTemperatureUnits.ABBR == "C"){
    return value;
  }
  if( gTemperatureUnits.ABBR == "K"){
    return value - 273.15;
  }
  if( gTemperatureUnits.ABBR == "F"){
    return (value - 32) * (5 / 9);
  }	
	return 0.0;
}

function toKelvin(value) {
  if( gTemperatureUnits.ABBR == "C"){
    return value  + 273.15;
  }
  if( gTemperatureUnits.ABBR == "K"){
    return value;
  }
  if( gTemperatureUnits.ABBR == "F"){
    return (value + 459.67) * (5 / 9);
  }	
	return 0.0;
}

function toFahrenheit(value) {
  if( gTemperatureUnits.ABBR == "C"){
    return (value * (9/5))  + 32;
  }
  if( gTemperatureUnits.ABBR == "K"){
    return (value * (9/5)) - 459.67;
  }
  if( gTemperatureUnits.ABBR == "F"){
    return value;
  }	
	return 0.0;
}

// pressure units conversions
function toHPascal(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa;
}

function toMilliBar(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa;
}

function toMilliMercury(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa * 0.75006150504;
}

function toMilliMeterMercury(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa * 0.75006150504;
}

function toInchMercury(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa * 0.029529980514;
}

function toInMercury(value) {
	var hPa = value * gPressureUnits.FACTOR_INT;
	return hPa * 0.029529980514;
}


// LandXmlExport export to different units

]]>
</msxsl:script>    

  
</xsl:stylesheet>
